Attribute VB_Name = "Registry"
Option Explicit

Public Const HKEY_CLASSES_ROOT  As Long = &H80000000
Public Const HKEY_CURRENT_USER  As Long = &H80000001
Public Const HKEY_LOCAL_MACHINE As Long = &H80000002

Const REG_OPTION_NON_VOLATILE   As Long = 0
Const KEY_SET_VALUE             As Long = &H2
Const ERROR_SUCCESS             As Long = 0
Const REG_SZ                    As Long = 1
Const KEY_QUERY_VALUE           As Long = &H1
Const KEY_ENUMERATE_SUB_KEYS    As Long = &H8

' Structures Needed For Registry Prototypes
Private Type SECURITY_ATTRIBUTES
    nLength As Long
    lpSecurityDescriptor As Long
    bInheritHandle As Boolean
End Type

Type FILETIME
    dwLowDateTime As Long
    dwHighDateTime As Long
End Type

Private Declare Function RegCloseKey Lib "advapi32.dll" (ByVal hKey As Long) As Long
Private Declare Function RegCreateKey Lib "advapi32.dll" Alias "RegCreateKeyA" (ByVal hKey As Long, ByVal lpSubKey As String, phkResult As Long) As Long
Private Declare Function RegCreateKeyEx Lib "advapi32.dll" Alias "RegCreateKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal Reserved As Long, ByVal lpClass As String, ByVal dwOptions As Long, ByVal samDesired As Long, lpSecurityAttributes As SECURITY_ATTRIBUTES, phkResult As Long, lpdwDisposition As Long) As Long
Private Declare Function RegDeleteKey Lib "advapi32.dll" Alias "RegDeleteKeyA" (ByVal hKey As Long, ByVal lpSubKey As String) As Long
Private Declare Function RegDeleteValue Lib "advapi32.dll" Alias "RegDeleteValueA" (ByVal hKey As Long, ByVal lpValueName As String) As Long
Private Declare Function RegEnumKey Lib "advapi32.dll" Alias "RegEnumKeyA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpName As String, ByVal cbName As Long) As Long
Private Declare Function RegEnumKeyEx Lib "advapi32.dll" Alias "RegEnumKeyExA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpName As String, lpcbName As Long, lpReserved As Long, ByVal lpClass As String, lpcbClass As Long, lpftLastWriteTime As FILETIME) As Long
Private Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal ulOptions As Long, ByVal samDesired As Long, phkResult As Long) As Long
Private Declare Function RegQueryValueEx Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, lpType As Long, lpData As Any, lpcbData As Long) As Long
Private Declare Function RegSetValueEx Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal Reserved As Long, ByVal dwType As Long, ByVal lpData As String, ByVal cbData As Long) As Long

Public Function GetRegistryString(hKey As Long, sSubKey As String, _
                sValueName As String, sValue As String, sDefault As String) As Boolean
    
    Dim iNullPosition As Integer
    
    GetRegistryString = False
    
    Dim hkResult As Long
    Dim lBufferSize As Long
    lBufferSize = Len(sValue)
    
    If (RegOpenKeyEx(hKey, sSubKey, 0, KEY_QUERY_VALUE, hkResult)) = ERROR_SUCCESS Then
        If (RegQueryValueEx(hkResult, sValueName, 0, 0, ByVal sValue, lBufferSize)) = ERROR_SUCCESS Then
            If lBufferSize > 0 Then
                iNullPosition = InStr(sValue, Chr(0))     ' Locate position of NULL
                If iNullPosition > 0 Then                 ' Remove the NULL if found
                    frmRegistry.txtFromRegistry = Left(sValue, iNullPosition - 1)
                Else                        ' Otherwise set the value to an empty string
                    sValue = ""
                End If
            End If
        Else
            sValue = ""
        End If
        GetRegistryString = True
    End If
    Call RegCloseKey(hkResult)

End Function

Public Function SetRegistryString(hKey As Long, sSubKey As String, _
                                  sValueName As String, sValue As String) As Boolean
                                  
    SetRegistryString = False

    Dim phkResult           As Long
    Dim lpdwDisposition     As Long
    Dim SecurityAttributes  As SECURITY_ATTRIBUTES
    Dim lLength             As Long
    
    If (RegCreateKeyEx(hKey, sSubKey, 0, "", REG_OPTION_NON_VOLATILE, _
        KEY_SET_VALUE, SecurityAttributes, phkResult, lpdwDisposition)) = ERROR_SUCCESS Then
                
        lLength = Len(sValue)
        If lLength = 0 Then
            sValue = ""
        End If
        If (RegSetValueEx(phkResult, sValueName, 0, REG_SZ, sValue, lLength + 1)) = ERROR_SUCCESS Then
            SetRegistryString = True
        End If
        
    End If
    Call RegCloseKey(phkResult)
        
End Function

Public Function CreateRegistryKey(hKey As Long, sSubKey As String) As Boolean
    'hKey is the handle and sSubKey is the key to create
    Dim hkResult As Long
    Dim lpdwDisposition As Long
    Dim SecurityAttributes As SECURITY_ATTRIBUTES
    'Dim lLength As Long
    'Dim sTempValue As String
    
    CreateRegistryKey = False

    If (RegCreateKeyEx(hKey, sSubKey, 0, "", REG_OPTION_NON_VOLATILE, _
        KEY_SET_VALUE, SecurityAttributes, hkResult, lpdwDisposition)) = ERROR_SUCCESS Then
        Call RegCloseKey(hkResult)
        CreateRegistryKey = True
    End If
        
End Function

Public Function DeleteRegistryKey(hKey As Long, sSubKey As String, sKey As String) As Boolean
        
    Dim hkResult As Long
    DeleteRegistryKey = False

    If (RegOpenKeyEx(hKey, sSubKey, 0, KEY_QUERY_VALUE, hkResult)) = ERROR_SUCCESS Then
        If RegDeleteKey(hkResult, sKey) = ERROR_SUCCESS Then
        DeleteRegistryKey = True
        End If
    End If
        
End Function


